@@ -1,4 +1,4 @@
- // -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
+// -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
 // vim: ts=8 sw=2 smarttab
 /*
  * Ceph - scalable distributed file system
@@ -16,6 +16,7 @@
 #include "common/code_environment.h"
 #include "common/debug.h"
 #include "common/signal.h"
+#include "common/io_priority.h"
 
 #include <dirent.h>
 #include <errno.h>
@@ -26,10 +27,34 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
+#ifdef HAVE_SCHED
+#include <sched.h>
+#endif
 
+static int _set_affinity(int id)
+{
+#ifdef HAVE_SCHED
+  if (id >= 0 && id < CPU_SETSIZE) {
+    cpu_set_t cpuset;
+    CPU_ZERO(&cpuset);
+
+    CPU_SET(id, &cpuset);
+
+    if (sched_setaffinity(0, sizeof(cpuset), &cpuset) < 0)
+      return -errno;
+    /* guaranteed to take effect immediately */
+    sched_yield();
+  }
+#endif
+  return 0;
+}
 
 Thread::Thread()
-  : thread_id(0)
+  : thread_id(0),
+    pid(0),
+    ioprio_class(-1),
+    ioprio_priority(-1),
+    cpuid(-1)
 {
 }
 
@@ -38,21 +63,38 @@ Thread::~Thread()
 }
 
 void *Thread::_entry_func(void *arg) {
-  void *r = ((Thread*)arg)->entry();
+  void *r = ((Thread*)arg)->entry_wrapper();
   return r;
 }
 
-const pthread_t &Thread::get_thread_id()
+void *Thread::entry_wrapper()
+{
+  int p = ceph_gettid(); // may return -ENOSYS on other platforms
+  if (p > 0)
+    pid = p;
+  if (pid &&
+      ioprio_class >= 0 &&
+      ioprio_priority >= 0) {
+    ceph_ioprio_set(IOPRIO_WHO_PROCESS,
+		    pid,
+		    IOPRIO_PRIO_VALUE(ioprio_class, ioprio_priority));
+  }
+  if (pid && cpuid >= 0)
+    _set_affinity(cpuid);
+  return entry();
+}
+
+const pthread_t &Thread::get_thread_id() const
 {
   return thread_id;
 }
 
-bool Thread::is_started()
+bool Thread::is_started() const
 {
   return thread_id != 0;
 }
 
-bool Thread::am_self()
+bool Thread::am_self() const
 {
   return (pthread_self() == thread_id);
 }
@@ -119,7 +161,14 @@ int Thread::join(void **prval)
   }
 
   int status = pthread_join(thread_id, prval);
-  assert(status == 0);
+  if (status != 0) {
+    char buf[256];
+    snprintf(buf, sizeof(buf), "Thread::join(): pthread_join "
+             "failed with error %d\n", status);
+    dout_emergency(buf);
+    assert(status == 0);
+  }
+
   thread_id = 0;
   return status;
 }
@@ -128,3 +177,23 @@ int Thread::detach()
 {
   return pthread_detach(thread_id);
 }
+
+int Thread::set_ioprio(int cls, int prio)
+{
+  // fixme, maybe: this can race with create()
+  ioprio_class = cls;
+  ioprio_priority = prio;
+  if (pid && cls >= 0 && prio >= 0)
+    return ceph_ioprio_set(IOPRIO_WHO_PROCESS,
+			   pid,
+			   IOPRIO_PRIO_VALUE(cls, prio));
+  return 0;
+}
+
+int Thread::set_affinity(int id)
+{
+  cpuid = id;
+  if (pid && ceph_gettid() == pid)
+    _set_affinity(id);
+  return 0;
+}
