@@ -130,15 +130,6 @@ void hobject_t::decode(bufferlist::iterator& bl)
   if (struct_v >= 4) {
     ::decode(nspace, bl);
     ::decode(pool, bl);
-    // newer OSDs have a different hobject_t::get_min(); decode it properly.
-    if (pool == INT64_MIN &&
-	hash == 0 &&
-	snap == 0 &&
-	!max &&
-	oid.name.empty()) {
-      pool = -1;
-      assert(is_min());
-    }
   }
   DECODE_FINISH(bl);
   build_filestore_key_cache();
@@ -193,13 +184,17 @@ void hobject_t::generate_test_instances(list<hobject_t*>& o)
 
 ostream& operator<<(ostream& out, const hobject_t& o)
 {
+  if (o == hobject_t())
+    return out << "MIN";
   if (o.is_max())
     return out << "MAX";
+  out << o.pool << '/';
   out << std::hex << o.get_hash() << std::dec;
+  if (o.nspace.length())
+    out << ":" << o.nspace;
   if (o.get_key().length())
     out << "." << o.get_key();
   out << "/" << o.oid << "/" << o.snap;
-  out << "/" << o.nspace << "/" << o.pool;
   return out;
 }
 
@@ -207,7 +202,7 @@ ostream& operator<<(ostream& out, const hobject_t& o)
 // version 5.
 void ghobject_t::encode(bufferlist& bl) const
 {
-  ENCODE_START(5, 3, bl);
+  ENCODE_START(6, 3, bl);
   ::encode(hobj.key, bl);
   ::encode(hobj.oid, bl);
   ::encode(hobj.snap, bl);
@@ -217,12 +212,13 @@ void ghobject_t::encode(bufferlist& bl) const
   ::encode(hobj.pool, bl);
   ::encode(generation, bl);
   ::encode(shard_id, bl);
+  ::encode(max, bl);
   ENCODE_FINISH(bl);
 }
 
 void ghobject_t::decode(bufferlist::iterator& bl)
 {
-  DECODE_START_LEGACY_COMPAT_LEN(5, 3, 3, bl);
+  DECODE_START_LEGACY_COMPAT_LEN(6, 3, 3, bl);
   if (struct_v >= 1)
     ::decode(hobj.key, bl);
   ::decode(hobj.oid, bl);
@@ -235,15 +231,6 @@ void ghobject_t::decode(bufferlist::iterator& bl)
   if (struct_v >= 4) {
     ::decode(hobj.nspace, bl);
     ::decode(hobj.pool, bl);
-    // newer OSDs have a different hobject_t::get_min(); decode it properly.
-    if (hobj.pool == INT64_MIN &&
-	hobj.hash == 0 &&
-	hobj.snap == 0 &&
-	!hobj.max &&
-	hobj.oid.name.empty()) {
-      hobj.pool = -1;
-      assert(hobj.is_min());
-    }
   }
   if (struct_v >= 5) {
     ::decode(generation, bl);
@@ -252,6 +239,11 @@ void ghobject_t::decode(bufferlist::iterator& bl)
     generation = ghobject_t::NO_GEN;
     shard_id = shard_id_t::NO_SHARD;
   }
+  if (struct_v >= 6) {
+    ::decode(max, bl);
+  } else {
+    max = false;
+  }
   DECODE_FINISH(bl);
   hobj.set_hash(hobj.get_hash()); //to call build_filestore_key_cache();
 }
@@ -267,6 +259,8 @@ void ghobject_t::decode(json_spirit::Value& v)
       generation = p.value_.get_uint64();
     else if (p.name_ == "shard_id")
       shard_id.id = p.value_.get_int();
+    else if (p.name_ == "max")
+      max = p.value_.get_int();
   }
 }
 
@@ -277,6 +271,7 @@ void ghobject_t::dump(Formatter *f) const
     f->dump_int("generation", generation);
   if (shard_id != shard_id_t::NO_SHARD)
     f->dump_int("shard_id", shard_id);
+  f->dump_int("max", (int)max);
 }
 
 void ghobject_t::generate_test_instances(list<ghobject_t*>& o)
@@ -306,11 +301,15 @@ void ghobject_t::generate_test_instances(list<ghobject_t*>& o)
 
 ostream& operator<<(ostream& out, const ghobject_t& o)
 {
+  if (o == ghobject_t())
+    return out << "GHMIN";
+  if (o.is_max())
+    return out << "GHMAX";
+  if (o.shard_id != shard_id_t::NO_SHARD)
+    out << std::hex << o.shard_id << std::dec << ":";
   out << o.hobj;
-  if (o.generation != ghobject_t::NO_GEN ||
-      o.shard_id != shard_id_t::NO_SHARD) {
-    assert(o.shard_id != shard_id_t::NO_SHARD);
-    out << "/" << o.generation << "/" << (unsigned)(o.shard_id);
+  if (o.generation != ghobject_t::NO_GEN) {
+    out << "/" << std::hex << (unsigned)(o.generation) << std::dec;
   }
   return out;
 }
