@@ -29,13 +29,23 @@ namespace ceph {
   class Formatter;
 }
 
+#ifndef UINT64_MAX
+#define UINT64_MAX (18446744073709551615ULL)
+#endif
+#ifndef INT64_MIN
+#define INT64_MIN ((int64_t)0x8000000000000000ll)
+#endif
+
 struct hobject_t {
   object_t oid;
   snapid_t snap;
-  uint32_t hash;
 private:
+  uint32_t hash;
   bool max;
-  static const int64_t POOL_IS_TEMP = -1;
+  filestore_hobject_key_t filestore_key_cache;
+  static const int64_t POOL_META = -1;
+  static const int64_t POOL_TEMP_START = -2; // and then negative
+  friend class spg_t;  // for POOL_TEMP_START
 public:
   int64_t pool;
   string nspace;
@@ -48,7 +58,19 @@ public:
     return key;
   }
 
+  void set_key(const std::string &key_) {
+    key = key_;
+  }
+
   string to_str() const;
+  
+  uint32_t get_hash() const { 
+    return hash;
+  }
+  void set_hash(uint32_t value) { 
+    hash = value;
+    build_filestore_key_cache();
+  }
 
   static bool match_hash(uint32_t to_check, uint32_t bits, uint32_t match) {
     return (match & ~((~0)<<bits)) == (to_check & ~((~0)<<bits));
@@ -57,34 +79,40 @@ public:
     return match_hash(hash, bits, match);
   }
 
-  static hobject_t make_temp(const string &name) {
-    hobject_t ret(object_t(name), "", CEPH_NOSNAP, 0, POOL_IS_TEMP, "");
-    return ret;
-  }
   bool is_temp() const {
-    return pool == POOL_IS_TEMP;
+    return pool < POOL_TEMP_START && pool != INT64_MIN;
+  }
+  bool is_meta() const {
+    return pool == POOL_META;
+  }
+
+  hobject_t() : snap(0), hash(0), max(false), pool(INT64_MIN) {
+    build_filestore_key_cache();
   }
-  
-  hobject_t() : snap(0), hash(0), max(false), pool(-1) {}
 
   hobject_t(object_t oid, const string& key, snapid_t snap, uint64_t hash,
-	    int64_t pool, string nspace) :
-    oid(oid), snap(snap), hash(hash), max(false),
-    pool(pool), nspace(nspace),
-    key(oid.name == key ? string() : key) {}
+	    int64_t pool, string nspace)
+    : oid(oid), snap(snap), hash(hash), max(false),
+      pool(pool), nspace(nspace),
+      key(oid.name == key ? string() : key) {
+    build_filestore_key_cache();
+  }
 
   hobject_t(const sobject_t &soid, const string &key, uint32_t hash,
-	    int64_t pool, string nspace) :
-    oid(soid.oid), snap(soid.snap), hash(hash), max(false),
-    pool(pool), nspace(nspace),
-    key(soid.oid.name == key ? string() : key) {}
+	    int64_t pool, string nspace)
+    : oid(soid.oid), snap(soid.snap), hash(hash), max(false),
+      pool(pool), nspace(nspace),
+      key(soid.oid.name == key ? string() : key) {
+    build_filestore_key_cache();
+  }
 
   /// @return min hobject_t ret s.t. ret.hash == this->hash
   hobject_t get_boundary() const {
     if (is_max())
       return *this;
     hobject_t ret;
-    ret.hash = hash;
+    ret.set_hash(hash);
+    ret.pool = pool;
     return ret;
   }
 
@@ -124,8 +152,8 @@ public:
 
   /* Do not use when a particular hash function is needed */
   explicit hobject_t(const sobject_t &o) :
-    oid(o.oid), snap(o.snap), max(false), pool(-1) {
-    hash = CEPH_HASH_NAMESPACE::hash<sobject_t>()(o);
+    oid(o.oid), snap(o.snap), max(false), pool(POOL_META) {
+    set_hash(CEPH_HASH_NAMESPACE::hash<sobject_t>()(o));
   }
 
   // maximum sorted value.
@@ -142,7 +170,7 @@ public:
     return snap == 0 &&
 	   hash == 0 &&
 	   !max &&
-	   pool == -1;
+	   pool == INT64_MIN;
   }
 
   static uint32_t _reverse_nibbles(uint32_t retval) {
@@ -156,8 +184,8 @@ public:
   /**
    * Returns set S of strings such that for any object
    * h where h.match(bits, mask), there is some string
-   * s \in S such that s is a prefix of h.to_str().
-   * Furthermore, for any s \in S, s is a prefix of
+   * s \f$\in\f$ S such that s is a prefix of h.to_str().
+   * Furthermore, for any s $f\in\f$ S, s is a prefix of
    * h.str() implies that h.match(bits, mask).
    */
   static set<string> get_prefixes(
@@ -170,10 +198,10 @@ public:
     return _reverse_nibbles(hash);
   }
   filestore_hobject_key_t get_filestore_key() const {
-    if (max)
-      return 0x100000000ull;
-    else
-      return get_filestore_key_u32();
+    return max ? 0x100000000ull : filestore_key_cache;
+  }
+  void build_filestore_key_cache() {    
+    filestore_key_cache = _reverse_nibbles(hash);
   }
 
   const string& get_effective_key() const {
@@ -219,36 +247,52 @@ CEPH_HASH_NAMESPACE_END
 
 ostream& operator<<(ostream& out, const hobject_t& o);
 
-WRITE_EQ_OPERATORS_7(hobject_t, oid, get_key(), snap, hash, max, pool, nspace)
-// sort hobject_t's by <max, get_filestore_key(hash), key, oid, snapid>
+WRITE_EQ_OPERATORS_7(hobject_t, hash, oid, get_key(), snap, pool, max, nspace)
 WRITE_CMP_OPERATORS_7(hobject_t,
 		      max,
+		      pool,
 		      get_filestore_key(),
 		      nspace,
-		      pool,
 		      get_effective_key(),
 		      oid,
 		      snap)
 
 typedef version_t gen_t;
 
-#ifndef UINT64_MAX
-#define UINT64_MAX (18446744073709551615ULL)
-#endif
-
 struct ghobject_t {
   hobject_t hobj;
   gen_t generation;
   shard_id_t shard_id;
+  bool max;
 
 public:
   static const gen_t NO_GEN = UINT64_MAX;
 
-  ghobject_t() : generation(NO_GEN), shard_id(shard_id_t::NO_SHARD) {}
+  ghobject_t()
+    : generation(NO_GEN),
+      shard_id(shard_id_t::NO_SHARD),
+      max(false) {}
 
-  ghobject_t(const hobject_t &obj) : hobj(obj), generation(NO_GEN), shard_id(shard_id_t::NO_SHARD) {}
+  explicit ghobject_t(const hobject_t &obj)
+    : hobj(obj),
+      generation(NO_GEN),
+      shard_id(shard_id_t::NO_SHARD),
+      max(false) {}
 
-  ghobject_t(const hobject_t &obj, gen_t gen, shard_id_t shard) : hobj(obj), generation(gen), shard_id(shard) {}
+  ghobject_t(const hobject_t &obj, gen_t gen, shard_id_t shard)
+    : hobj(obj),
+      generation(gen),
+      shard_id(shard),
+      max(false) {}
+
+  static ghobject_t make_pgmeta(int64_t pool, uint32_t hash, shard_id_t shard) {
+    hobject_t h(object_t(), string(), CEPH_NOSNAP, hash, pool, string());
+    return ghobject_t(h, NO_GEN, shard);
+  }
+  bool is_pgmeta() const {
+    // make sure we are distinct from hobject_t(), which has pool INT64_MIN
+    return hobj.pool >= 0 && hobj.oid.name.empty();
+  }
 
   bool match(uint32_t bits, uint32_t match) const {
     return hobj.match_hash(hobj.hash, bits, match);
@@ -258,7 +302,9 @@ public:
     if (hobj.is_max())
       return *this;
     ghobject_t ret;
-    ret.hobj.hash = hobj.hash;
+    ret.hobj.set_hash(hobj.hash);
+    ret.shard_id = shard_id;
+    ret.hobj.pool = hobj.pool;
     return ret;
   }
   filestore_hobject_key_t get_filestore_key_u32() const {
@@ -280,13 +326,22 @@ public:
     return shard_id == shard_id_t::NO_SHARD;
   }
 
+  void set_shard(shard_id_t s) {
+    shard_id = s;
+  }
+
   // maximum sorted value.
   static ghobject_t get_max() {
-    ghobject_t h(hobject_t::get_max());
+    ghobject_t h;
+    h.max = true;
+    h.hobj = hobject_t::get_max();  // so that is_max() => hobj.is_max()
     return h;
   }
   bool is_max() const {
-    return hobj.is_max();
+    return max;
+  }
+  bool is_min() const {
+    return *this == ghobject_t();
   }
 
   void swap(ghobject_t &o) {
@@ -321,14 +376,11 @@ CEPH_HASH_NAMESPACE_END
 
 ostream& operator<<(ostream& out, const ghobject_t& o);
 
-WRITE_EQ_OPERATORS_3(ghobject_t, hobj, shard_id, generation)
-// sort ghobject_t's by <hobj, shard_id, generation> 
-// 
-// Two objects which differ by generation are more related than
-// two objects of the same generation which differ by shard.
-// 
-WRITE_CMP_OPERATORS_3(ghobject_t,
-		      hobj,
+WRITE_EQ_OPERATORS_4(ghobject_t, max, shard_id, hobj, generation)
+
+WRITE_CMP_OPERATORS_4(ghobject_t,
+		      max,
 		      shard_id,
+		      hobj,
 		      generation)
 #endif
