@@ -35,6 +35,16 @@ MDS *MDSInternalContext::get_mds() {
   return mds;
 }
 
+MDS *MDSInternalContextWrapper::get_mds()
+{
+  return mds;
+}
+
+void MDSInternalContextWrapper::finish(int r)
+{
+  fin->complete(r);
+}
+
 
 void MDSIOContextBase::complete(int r) {
   MDS *mds = get_mds();
@@ -42,6 +52,12 @@ void MDSIOContextBase::complete(int r) {
   dout(10) << "MDSIOContextBase::complete: " << typeid(*this).name() << dendl;
   assert(mds != NULL);
   Mutex::Locker l(mds->mds_lock);
+  if (mds->stopping) {
+    dout(4) << "MDSIOContextBase::complete: dropping for stopping "
+            << typeid(*this).name() << dendl;
+    return;
+  }
+
   if (r == -EBLACKLISTED) {
     derr << "MDSIOContextBase: blacklisted!  Restarting..." << dendl;
     mds->respawn();
@@ -54,6 +70,15 @@ MDS *MDSIOContext::get_mds() {
   return mds;
 }
 
+MDS *MDSIOContextWrapper::get_mds() {
+  return mds;
+}
+
+void MDSIOContextWrapper::finish(int r)
+{
+  fin->complete(r);
+}
+
 MDS *MDSInternalContextGather::get_mds()
 {
   derr << "Forbidden call to MDSInternalContextGather::get_mds by " << typeid(*this).name() << dendl;
