@@ -35,7 +35,7 @@ struct MRoute : public Message {
   MRoute(bufferlist bl, const entity_inst_t& i)
     : Message(MSG_ROUTE, HEAD_VERSION, COMPAT_VERSION), session_mon_tid(0), dest(i) {
     bufferlist::iterator p = bl.begin();
-    msg = decode_message(NULL, p);
+    msg = decode_message(NULL, 0, p);
   }
 private:
   ~MRoute() {
@@ -51,15 +51,17 @@ public:
       bool m;
       ::decode(m, p);
       if (m)
-	msg = decode_message(NULL, p);
+	msg = decode_message(NULL, 0, p);
     } else {
-      msg = decode_message(NULL, p);
+      msg = decode_message(NULL, 0, p);
     }
   }
   void encode_payload(uint64_t features) {
     ::encode(session_mon_tid, payload);
     ::encode(dest, payload);
     if (features & CEPH_FEATURE_MON_NULLROUTE) {
+      header.version = HEAD_VERSION;
+      header.compat_version = COMPAT_VERSION;
       bool m = msg ? true : false;
       ::encode(m, payload);
       if (msg)
