@@ -58,10 +58,12 @@ class MDSMonitor : public PaxosService {
     void finish(int r) {
       if (r >= 0)
 	mm->_updated(m);   // success
-      else if (r == -ECANCELED)
+      else if (r == -ECANCELED) {
+	mm->mon->no_reply(m);
 	m->put();
-      else
+      } else {
 	mm->dispatch((PaxosServiceMessage*)m);        // try again
+      }
     }
   };
 
@@ -72,6 +74,7 @@ class MDSMonitor : public PaxosService {
   // service methods
   void create_initial();
   void update_from_paxos(bool *need_bootstrap);
+  void init();
   void create_pending(); 
   void encode_pending(MonitorDBStore::TransactionRef t);
   // we don't require full versions; don't encode any.
@@ -97,28 +100,27 @@ class MDSMonitor : public PaxosService {
   void get_health(list<pair<health_status_t,string> >& summary,
 		  list<pair<health_status_t,string> > *detail) const;
   int fail_mds(std::ostream &ss, const std::string &arg);
-  void fail_mds_gid(uint64_t gid);
+  void fail_mds_gid(mds_gid_t gid);
 
   bool preprocess_command(MMonCommand *m);
   bool prepare_command(MMonCommand *m);
-  bool management_command(
-      std::string const &prefix,
-      map<string, cmd_vartype> &cmdmap,
-      std::stringstream &ss,
-      int &r);
-  bool filesystem_command(
+  int management_command(
       MMonCommand *m,
       std::string const &prefix,
       map<string, cmd_vartype> &cmdmap,
-      std::stringstream &ss,
-      int &r);
+      std::stringstream &ss);
+  int filesystem_command(
+      MMonCommand *m,
+      std::string const &prefix,
+      map<string, cmd_vartype> &cmdmap,
+      std::stringstream &ss);
 
   // beacons
   struct beacon_info_t {
     utime_t stamp;
     uint64_t seq;
   };
-  map<uint64_t, beacon_info_t> last_beacon;
+  map<mds_gid_t, beacon_info_t> last_beacon;
 
   bool try_standby_replay(MDSMap::mds_info_t& finfo, MDSMap::mds_info_t& ainfo);
 
@@ -131,16 +133,25 @@ public:
   void tick();     // check state, take actions
 
   void dump_info(Formatter *f);
+  int dump_metadata(const string& who, Formatter *f, ostream& err);
+  int print_nodes(Formatter *f);
 
   void check_subs();
   void check_sub(Subscription *sub);
 
 private:
+  void update_metadata(mds_gid_t gid, const Metadata& metadata);
+  void remove_from_metadata(MonitorDBStore::TransactionRef t);
+  int load_metadata(map<mds_gid_t, Metadata>& m);
+
   // MDS daemon GID to latest health state from that GID
   std::map<uint64_t, MDSHealth> pending_daemon_health;
   std::set<uint64_t> pending_daemon_health_rm;
 
+  map<mds_gid_t, Metadata> pending_metadata;
+
   int _check_pool(const int64_t pool_id, std::stringstream *ss) const;
+  mds_gid_t gid_from_arg(const std::string& arg, std::ostream& err);
 };
 
 #endif
