@@ -65,15 +65,23 @@ struct OpRequest : public TrackedOp {
   bool need_write_cap();
   bool need_class_read_cap();
   bool need_class_write_cap();
+  bool need_promote();
+  bool need_skip_promote();
   void set_read();
   void set_write();
   void set_cache();
   void set_class_read();
   void set_class_write();
   void set_pg_op();
+  void set_promote();
+  void set_skip_promote();
 
   void _dump(utime_t now, Formatter *f) const;
 
+  bool has_feature(uint64_t f) const {
+    return request->get_connection()->has_feature(f);
+  }
+
 private:
   Message *request; /// the logical request we are tracking
   osd_reqid_t reqid;
@@ -90,7 +98,7 @@ private:
   OpRequest(Message *req, OpTracker *tracker);
 
 protected:
-  void _dump_op_descriptor(ostream& stream) const;
+  void _dump_op_descriptor_unlocked(ostream& stream) const;
   void _unregistered();
 
 public:
@@ -99,6 +107,7 @@ public:
   }
   bool send_map_update;
   epoch_t sent_epoch;
+  bool hitset_inserted;
   Message *get_req() const { return request; }
   bool been_queued_for_pg() { return hit_flag_points & flag_queued_for_pg; }
   bool been_reached_pg() { return hit_flag_points & flag_reached_pg; }
@@ -127,40 +136,22 @@ public:
   }
 
   void mark_queued_for_pg() {
-    mark_event("queued_for_pg");
-    current = "queued for pg";
-    hit_flag_points |= flag_queued_for_pg;
-    latest_flag_point = flag_queued_for_pg;
+    mark_flag_point(flag_queued_for_pg, "queued_for_pg");
   }
   void mark_reached_pg() {
-    mark_event("reached_pg");
-    current = "reached pg";
-    hit_flag_points |= flag_reached_pg;
-    latest_flag_point = flag_reached_pg;
+    mark_flag_point(flag_reached_pg, "reached_pg");
   }
-  void mark_delayed(string s) {
-    mark_event(s);
-    current = s;
-    hit_flag_points |= flag_delayed;
-    latest_flag_point = flag_delayed;
+  void mark_delayed(const string& s) {
+    mark_flag_point(flag_delayed, s);
   }
   void mark_started() {
-    mark_event("started");
-    current = "started";
-    hit_flag_points |= flag_started;
-    latest_flag_point = flag_started;
+    mark_flag_point(flag_started, "started");
   }
-  void mark_sub_op_sent(string s) {
-    mark_event(s);
-    current = s;
-    hit_flag_points |= flag_sub_op_sent;
-    latest_flag_point = flag_sub_op_sent;
+  void mark_sub_op_sent(const string& s) {
+    mark_flag_point(flag_sub_op_sent, s);
   }
   void mark_commit_sent() {
-    mark_event("commit_sent");
-    current = "commit sent";
-    hit_flag_points |= flag_commit_sent;
-    latest_flag_point = flag_commit_sent;
+    mark_flag_point(flag_commit_sent, "commit_sent");
   }
 
   utime_t get_dequeued_time() const {
@@ -175,6 +166,10 @@ public:
   }
 
   typedef ceph::shared_ptr<OpRequest> Ref;
+
+private:
+  void set_rmw_flags(int flags);
+  void mark_flag_point(uint8_t flag, const string& s);
 };
 
 typedef OpRequest::Ref OpRequestRef;
