@@ -37,6 +37,9 @@
 #include "sys/stat.h"
 
 #define MAX_TEST 1000000
+#define FILENAME "bufferlist"
+
+static char cmd[128];
 
 TEST(Buffer, constructors) {
   bool ceph_buffer_track = get_env_bool("CEPH_BUFFER_TRACK");
@@ -150,15 +153,16 @@ TEST(Buffer, constructors) {
     EXPECT_THROW(buffer::create_zero_copy(len, -1, NULL), buffer::error_code);
 
     unsigned zc_len = 4;
-    ::unlink("testfile");
-    EXPECT_EQ(0, ::system("echo ABC > testfile"));
-    int fd = ::open("testfile", O_RDONLY);
+    ::unlink(FILENAME);
+    snprintf(cmd, sizeof(cmd), "echo ABC > %s", FILENAME);
+    EXPECT_EQ(0, ::system(cmd));
+    int fd = ::open(FILENAME, O_RDONLY);
     bufferptr ptr(buffer::create_zero_copy(zc_len, fd, NULL));
     EXPECT_EQ(zc_len, ptr.length());
     if (ceph_buffer_track)
       EXPECT_EQ(zc_len, (unsigned)buffer::get_total_alloc());
     ::close(fd);
-    ::unlink("testfile");
+    ::unlink(FILENAME);
   }
 #endif
   if (ceph_buffer_track)
@@ -178,14 +182,15 @@ class TestRawPipe : public ::testing::Test {
 protected:
   virtual void SetUp() {
     len = 4;
-    ::unlink("testfile");
-    EXPECT_EQ(0, ::system("echo ABC > testfile"));
-    fd = ::open("testfile", O_RDONLY);
+    ::unlink(FILENAME);
+    snprintf(cmd, sizeof(cmd), "echo ABC > %s", FILENAME);
+    EXPECT_EQ(0, ::system(cmd));
+    fd = ::open(FILENAME, O_RDONLY);
     assert(fd >= 0);
   }
   virtual void TearDown() {
     ::close(fd);
-    ::unlink("testfile");
+    ::unlink(FILENAME);
   }
   int fd;
   unsigned len;
@@ -295,21 +300,21 @@ TEST_F(TestRawPipe, buffer_list_read_fd_zero_copy) {
 }
 
 TEST_F(TestRawPipe, buffer_list_write_fd_zero_copy) {
-  ::unlink("testfile_out");
+  ::unlink(FILENAME);
   bufferlist bl;
   EXPECT_EQ(0, bl.read_fd_zero_copy(fd, len));
   EXPECT_TRUE(bl.can_zero_copy());
-  int out_fd = ::open("testfile_out", O_RDWR|O_CREAT|O_TRUNC, 0600);
+  int out_fd = ::open(FILENAME, O_RDWR|O_CREAT|O_TRUNC, 0600);
   EXPECT_EQ(0, bl.write_fd_zero_copy(out_fd));
   struct stat st;
   memset(&st, 0, sizeof(st));
-  EXPECT_EQ(0, ::stat("testfile_out", &st));
+  EXPECT_EQ(0, ::stat(FILENAME, &st));
   EXPECT_EQ(len, st.st_size);
   char buf[len + 1];
   EXPECT_EQ((int)len, safe_read(out_fd, buf, len + 1));
   EXPECT_EQ(0, memcmp(buf, "ABC\n", len));
   ::close(out_fd);
-  ::unlink("testfile_out");
+  ::unlink(FILENAME);
 }
 #endif // CEPH_HAVE_SPLICE
 
@@ -1077,22 +1082,52 @@ TEST(BufferList, buffers) {
 }
 
 TEST(BufferList, get_contiguous) {
-  bufferptr a("foobarbaz", 9);
-  bufferptr b("123456789", 9);
-  bufferptr c("ABCDEFGHI", 9);
-  bufferlist bl;
-  ASSERT_EQ(0, bl.get_contiguous(0, 0));
+  {
+    bufferptr a("foobarbaz", 9);
+    bufferptr b("123456789", 9);
+    bufferptr c("ABCDEFGHI", 9);
+    bufferlist bl;
+    ASSERT_EQ(0, bl.get_contiguous(0, 0));
 
-  bl.append(a);
-  bl.append(b);
-  bl.append(c);
-  ASSERT_EQ(3u, bl.buffers().size());
-  ASSERT_EQ(0, memcmp("bar", bl.get_contiguous(3, 3), 3));
-  ASSERT_EQ(0, memcmp("456", bl.get_contiguous(12, 3), 3));
-  ASSERT_EQ(0, memcmp("ABC", bl.get_contiguous(18, 3), 3));
-  ASSERT_EQ(3u, bl.buffers().size());
-  ASSERT_EQ(0, memcmp("789ABC", bl.get_contiguous(15, 6), 6));
-  ASSERT_LT(bl.buffers().size(), 3u);
+    bl.append(a);
+    bl.append(b);
+    bl.append(c);
+    ASSERT_EQ(3u, bl.buffers().size());
+    ASSERT_EQ(0, memcmp("bar", bl.get_contiguous(3, 3), 3));
+    ASSERT_EQ(0, memcmp("456", bl.get_contiguous(12, 3), 3));
+    ASSERT_EQ(0, memcmp("ABC", bl.get_contiguous(18, 3), 3));
+    ASSERT_EQ(3u, bl.buffers().size());
+    ASSERT_EQ(0, memcmp("789ABC", bl.get_contiguous(15, 6), 6));
+    ASSERT_EQ(2u, bl.buffers().size());
+  }
+
+  {
+    bufferptr a("foobarbaz", 9);
+    bufferptr b("123456789", 9);
+    bufferptr c("ABCDEFGHI", 9);
+    bufferlist bl;
+
+    bl.append(a);
+    bl.append(b);
+    bl.append(c);
+
+    ASSERT_EQ(0, memcmp("789ABCDEFGHI", bl.get_contiguous(15, 12), 12));
+    ASSERT_EQ(2u, bl.buffers().size());
+  }
+
+  {
+    bufferptr a("foobarbaz", 9);
+    bufferptr b("123456789", 9);
+    bufferptr c("ABCDEFGHI", 9);
+    bufferlist bl;
+
+    bl.append(a);
+    bl.append(b);
+    bl.append(c);
+
+    ASSERT_EQ(0, memcmp("z123456789AB", bl.get_contiguous(8, 12), 12));
+    ASSERT_EQ(1u, bl.buffers().size());
+  }
 }
 
 TEST(BufferList, swap) {
@@ -1830,6 +1865,18 @@ TEST(BufferList, splice) {
   bl.splice(4, 4);
   EXPECT_EQ((unsigned)4, bl.length());
   EXPECT_EQ(0, ::memcmp("ABCD", bl.c_str(), bl.length()));
+
+  {
+    bl.clear();
+    bufferptr ptr1("0123456789", 10);
+    bl.push_back(ptr1);
+    bufferptr ptr2("abcdefghij", 10);
+    bl.append(ptr2, 5, 5);
+    other.clear();
+    bl.splice(10, 4, &other);
+    EXPECT_EQ((unsigned)11, bl.length());
+    EXPECT_EQ(0, ::memcmp("fghi", other.c_str(), other.length()));
+  }
 }
 
 TEST(BufferList, write) {
@@ -1874,13 +1921,16 @@ TEST(BufferList, hexdump) {
 TEST(BufferList, read_file) {
   std::string error;
   bufferlist bl;
-  ::unlink("testfile");
+  ::unlink(FILENAME);
   EXPECT_EQ(-ENOENT, bl.read_file("UNLIKELY", &error));
-  EXPECT_EQ(0, ::system("echo ABC > testfile ; chmod 0 testfile"));
-  EXPECT_EQ(-EACCES, bl.read_file("testfile", &error));
-  EXPECT_EQ(0, ::system("chmod +r testfile"));
-  EXPECT_EQ(0, bl.read_file("testfile", &error));
-  ::unlink("testfile");
+  snprintf(cmd, sizeof(cmd), "echo ABC > %s ; chmod 0 %s", FILENAME, FILENAME);
+  EXPECT_EQ(0, ::system(cmd));
+  if (getuid() != 0)
+    EXPECT_EQ(-EACCES, bl.read_file(FILENAME, &error));
+  snprintf(cmd, sizeof(cmd), "chmod +r %s", FILENAME);
+  EXPECT_EQ(0, ::system(cmd));
+  EXPECT_EQ(0, bl.read_file(FILENAME, &error));
+  ::unlink(FILENAME);
   EXPECT_EQ((unsigned)4, bl.length());
   std::string actual(bl.c_str(), bl.length());
   EXPECT_EQ("ABC\n", actual);
@@ -1888,36 +1938,37 @@ TEST(BufferList, read_file) {
 
 TEST(BufferList, read_fd) {
   unsigned len = 4;
-  ::unlink("testfile");
-  EXPECT_EQ(0, ::system("echo ABC > testfile"));
+  ::unlink(FILENAME);
+  snprintf(cmd, sizeof(cmd), "echo ABC > %s", FILENAME);
+  EXPECT_EQ(0, ::system(cmd));
   int fd = -1;
   bufferlist bl;
   EXPECT_EQ(-EBADF, bl.read_fd(fd, len));
-  fd = ::open("testfile", O_RDONLY);
+  fd = ::open(FILENAME, O_RDONLY);
   EXPECT_EQ(len, (unsigned)bl.read_fd(fd, len));
   EXPECT_EQ(CEPH_PAGE_SIZE - len, bl.buffers().front().unused_tail_length());
   EXPECT_EQ(len, bl.length());
   ::close(fd);
-  ::unlink("testfile");
+  ::unlink(FILENAME);
 }
 
 TEST(BufferList, write_file) {
-  ::unlink("testfile");
+  ::unlink(FILENAME);
   int mode = 0600;
   bufferlist bl;
   EXPECT_EQ(-ENOENT, bl.write_file("un/like/ly", mode));
   bl.append("ABC");
-  EXPECT_EQ(0, bl.write_file("testfile", mode));
+  EXPECT_EQ(0, bl.write_file(FILENAME, mode));
   struct stat st;
   memset(&st, 0, sizeof(st));
-  ::stat("testfile", &st);
+  ::stat(FILENAME, &st);
   EXPECT_EQ((unsigned)(mode | S_IFREG), st.st_mode);
-  ::unlink("testfile");
+  ::unlink(FILENAME);
 }
 
 TEST(BufferList, write_fd) {
-  ::unlink("testfile");
-  int fd = ::open("testfile", O_WRONLY|O_CREAT|O_TRUNC, 0600);
+  ::unlink(FILENAME);
+  int fd = ::open(FILENAME, O_WRONLY|O_CREAT|O_TRUNC, 0600);
   bufferlist bl;
   for (unsigned i = 0; i < IOV_MAX * 2; i++) {
     bufferptr ptr("A", 1);
@@ -1927,9 +1978,9 @@ TEST(BufferList, write_fd) {
   ::close(fd);
   struct stat st;
   memset(&st, 0, sizeof(st));
-  ::stat("testfile", &st);
+  ::stat(FILENAME, &st);
   EXPECT_EQ(IOV_MAX * 2, st.st_size);
-  ::unlink("testfile");
+  ::unlink(FILENAME);
 }
 
 TEST(BufferList, crc32c) {
@@ -2212,6 +2263,17 @@ TEST(BufferList, zero) {
     bl.zero((unsigned)3, (unsigned)3);
     EXPECT_EQ(0, ::memcmp("ABC\0\0\0GHIKLM", bl.c_str(), 9));
   }
+  {
+    bufferlist bl;
+    bufferptr ptr1(4);
+    bufferptr ptr2(4);
+    memset(ptr1.c_str(), 'a', 4);
+    memset(ptr2.c_str(), 'b', 4);
+    bl.append(ptr1);
+    bl.append(ptr2);
+    bl.zero((unsigned)2, (unsigned)4);
+    EXPECT_EQ(0, ::memcmp("aa\0\0\0\0bb", bl.c_str(), 8));
+  }
 }
 
 TEST(BufferList, EmptyAppend) {
@@ -2323,6 +2385,46 @@ TEST(BufferList, TestCopyAll) {
   ASSERT_EQ(memcmp(big.get(), big2.get(), BIG_SZ), 0);
 }
 
+TEST(BufferList, InvalidateCrc) {
+  const static size_t buffer_size = 262144;
+  ceph::shared_ptr <unsigned char> big(
+      (unsigned char*)malloc(buffer_size), free);
+  unsigned char c = 0;
+  char* ptr = (char*) big.get();
+  char* inptr;
+  for (size_t i = 0; i < buffer_size; ++i) {
+    ptr[i] = c++;
+  }
+  bufferlist bl;
+  
+  // test for crashes (shouldn't crash)
+  bl.invalidate_crc();
+  
+  // put data into bufferlist
+  bl.append((const char*)big.get(), buffer_size);
+  
+  // get its crc
+  __u32 crc = bl.crc32c(0);
+  
+  // modify data in bl without its knowledge
+  inptr = (char*) bl.c_str();
+  c = 0;
+  for (size_t i = 0; i < buffer_size; ++i) {
+    inptr[i] = c--;
+  }
+  
+  // make sure data in bl are now different than in big
+  EXPECT_NE(memcmp((void*) ptr, (void*) inptr, buffer_size), 0);
+  
+  // crc should remain the same
+  __u32 new_crc = bl.crc32c(0);
+  EXPECT_EQ(crc, new_crc);
+  
+  // force crc invalidate, check if it is updated
+  bl.invalidate_crc();
+  EXPECT_NE(crc, bl.crc32c(0));
+}
+
 TEST(BufferHash, all) {
   {
     bufferlist bl;
