@@ -12,22 +12,21 @@
  *
  */
 
-#include "include/types.h"
-#include "include/atomic.h"
-
-#include "common/obj_bencher.h"
-#include "common/config.h"
-#include "common/debug.h"
-#include "common/ceph_argparse.h"
-#include "common/WorkQueue.h"
-#include "msg/Message.h"
-#include "global/global_init.h"
+#include <deque>
+#include <errno.h>
 
 #include "libs3.h"
 
-#include <deque>
+#include "common/ceph_argparse.h"
+#include "common/debug.h"
+#include "common/obj_bencher.h"
+#include "common/WorkQueue.h"
 
-#include <errno.h>
+#include "include/types.h"
+#include "include/atomic.h"
+
+#include "global/global_init.h"
+#include "msg/Message.h"
 
 #define DEFAULT_USER_AGENT "rest-bench"
 #define DEFAULT_BUCKET "rest-bench-bucket"
@@ -36,7 +35,7 @@ void usage(ostream& out)
 {
   out <<					\
 "usage: rest-bench [options] <write|seq>\n"
-"       rest-bench [options] cleanup <prefix>\n"
+"       rest-bench [options] cleanup [--run-name run_name] [--prefix prefix]\n"
 "BENCHMARK OPTIONS\n"
 "   --seconds\n"
 "        benchmak length (default: 60)\n"
@@ -282,6 +281,11 @@ public:
     list_bucket_handler.listBucketCallback = list_bucket_callback;
 
   }
+  ~RESTDispatcher()
+  {
+    req_wq.drain();
+    m_tp.stop();
+  } 
   void process_context(req_context *ctx);
   void get_obj(req_context *ctx);
   void put_obj(req_context *ctx);
@@ -678,6 +682,8 @@ int main(int argc, const char **argv)
 
   bool show_time = false;
   bool cleanup = true;
+  std::string run_name;
+  std::string prefix;
 
 
   for (i = args.begin(); i != args.end(); ) {
@@ -704,7 +710,7 @@ int main(int argc, const char **argv)
     } else if (ceph_argparse_witharg(args, i, &proto_str, "--protocol", (char*)NULL)) {
       if (strcasecmp(proto_str.c_str(), "http") == 0) {
         protocol = S3ProtocolHTTP;
-      } else if (strcasecmp(proto_str.c_str(), "http") == 0) {
+      } else if (strcasecmp(proto_str.c_str(), "https") == 0) {
         protocol = S3ProtocolHTTPS;
       } else {
         cerr << "bad protocol" << std::endl;
@@ -722,6 +728,10 @@ int main(int argc, const char **argv)
       }
     } else if (ceph_argparse_witharg(args, i, &val, "-t", "--concurrent-ios", (char*)NULL)) {
       concurrent_ios = strtol(val.c_str(), NULL, 10);
+    } else if (ceph_argparse_witharg(args, i, &val, "--run-name", (char*)NULL)) {
+      run_name = val;
+    } else if (ceph_argparse_witharg(args, i, &val, "--prefix", (char*)NULL)) {
+      prefix = val;
     } else if (ceph_argparse_witharg(args, i, &val, "--seconds", (char*)NULL)) {
       seconds = strtol(val.c_str(), NULL, 10);
     } else if (ceph_argparse_witharg(args, i, &val, "-b", "--block-size", (char*)NULL)) {
@@ -733,14 +743,9 @@ int main(int argc, const char **argv)
     }
   }
 
-  if (bucket.empty()) {
-    cerr << "rest-bench: bucket not specified" << std::endl;
-    usage_exit();
-  }
   if (args.empty())
     usage_exit();
   int operation = 0;
-  const char *prefix = NULL;
   if (strcmp(args[0], "write") == 0)
     operation = OP_WRITE;
   else if (strcmp(args[0], "seq") == 0)
@@ -748,10 +753,7 @@ int main(int argc, const char **argv)
   else if (strcmp(args[0], "rand") == 0)
     operation = OP_RAND_READ;
   else if (strcmp(args[0], "cleanup") == 0) {
-    if (args.size() < 2)
-      usage_exit();
     operation = OP_CLEANUP;
-    prefix = args[1];
   } else
     usage_exit();
 
@@ -784,12 +786,12 @@ int main(int argc, const char **argv)
   }
 
   if (operation == OP_CLEANUP) {
-    ret = bencher.clean_up(prefix, concurrent_ios);
+    ret = bencher.clean_up(prefix, concurrent_ios, run_name);
     if (ret != 0)
       cerr << "error during cleanup: " << ret << std::endl;
   } else {
-    ret = bencher.aio_bench(operation, seconds, 0,
-			    concurrent_ios, op_size, cleanup);
+    ret = bencher.aio_bench(operation, seconds,
+			    concurrent_ios, op_size, cleanup, run_name);
     if (ret != 0) {
         cerr << "error during benchmark: " << ret << std::endl;
     }
